# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents a class that defines the behavior of the edit
dialog.

@author: Lateef Alabi-Oki
@organization: The striim Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

class EditDialog(object):
	"""
	This class defines the behavior of the station manager's edit
	dialog.
	"""

	def __init__(self, manager, player, info):
		"""
		Initialize object.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player.
		@type player: A Player object.

		@param info: Radio station metadata.
		@type info: A Tuple object.
		"""
		self.__init_attributes(manager, player, info)
		self.__set_properties()
		self.__update_genre_combobox()
		self.__signal_id_1 = manager.connect("destroy", self.__destroy_cb)
		self.__signal_id_2 = self.__dialog.connect("delete-event", self.__delete_event_cb)
		self.__signal_id_3 = self.__dialog.connect("key-press-event", self.__key_press_event_cb)
		self.__signal_id_4 = self.__cancel_button.connect("clicked", self.__cancel_clicked_cb)
		self.__signal_id_5 = manager.connect("treeview-changed", self.__treeview_changed_cb)
		self.__signal_id_6 = self.__save_button.connect("clicked", self.__save_button_cb)
		# Monitor database for changes.
		from gnomevfs import monitor_add, MONITOR_FILE
		self.__monitor_id = monitor_add(self.__database_uri, MONITOR_FILE, self.__database_changed_cb)


	def __init_attributes(self, manager, player, info):
		"""
		Initialize data attributes.

		@param self: Reference to the EditDialog instance.
		@type self: A EditDialog object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param player: Reference to the Player.
		@type player: A Player object.

		@param info: Radio station metadata.
		@type info: A Tuple object.
		"""
		from os.path import join, split
		current_folder = split(globals()["__file__"])[0]
		glade_file = join(current_folder, "Stations.glade")
		from gtk.glade import XML
		glade = XML(glade_file, "EditDialog", domain="striim")
		self.__dialog = glade.get_widget("EditDialog")
		self.__save_button = glade.get_widget("EditDialogSaveButton")
		self.__cancel_button = glade.get_widget("EditDialogCancelButton")
		self.__name_entry = glade.get_widget("NameEntry")
		self.__genre_combobox = glade.get_widget("GenreComboBoxEntry")
		self.__genre_entry = glade.get_widget("GenreEntry")
		self.__station_label = glade.get_widget("StationNameLabel")
		self.__label_template = "<b><span color='blue'>%s</span></b>"
		self.__manager = manager
		self.__player = player
		self.__info = info
		# Path to the stations database.
		from Striim.Global import metadata_folder
		self.__database_path = metadata_folder + "/stations.gdb"
		from gnomevfs import get_uri_from_local_path
		self.__database_uri = get_uri_from_local_path(self.__database_path)
		return

	def show_dialog(self):
		"""
		Show dialog.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.
		"""
		self.__setup_widgets()
		self.__show()
		return

	def __setup_widgets(self):
		"""
		Initialize the widgets.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.
		"""
		station_name, name, genre = self.__get_info_from_database()
		from xml.sax.saxutils import escape
		self.__station_label.set_markup(self.__label_template % escape(station_name))
		self.__name_entry.set_text(name)
		self.__genre_entry.set_text(genre)
		self.__name_entry.grab_focus()
		return

	def __get_info_from_database(self):
		"""
		Get radio station information from database.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.

		@return: Radio station information.
		@rtype: A Tuple object.
		"""
		from Metadata import get_stations_from_database
		stations = get_stations_from_database()
		station_name = self.__info[0]
		name, genre, uris = stations[station_name]
		return station_name, name, genre

	def __update_database(self):
		"""
		Update the stations database.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.
		"""
		from Metadata import get_stations_from_database
		stations = get_stations_from_database()
		station_name = self.__info[0]
		name, genre, uris = stations[station_name]
		name = self.__name_entry.get_text()
		genre = self.__genre_entry.get_text()
		dictionary = {station_name.strip(): (name.strip(), genre.strip(), uris)}
		stations.update(dictionary)
		self.__manager.emit("station-edited", station_name.strip())
		from Metadata import update_stations_database
		update_stations_database(stations)
		return

	def __update_genre_combobox(self):
		"""
		Update the Genre combobox.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.
		"""
		from Metadata import get_stations_from_database
		stations = get_stations_from_database()
		model = self.__genre_combobox.get_model()
		model.clear()
		genres = set([])
		from operator import not_
		if not_(stations): return
		for name, genre, uris in stations.values():
			if genre: genres.add(genre)
		genres = list(genres)
		genres.sort()
		for genre in genres:
			model.append([genre])
		return

	def __show(self):
		"""
		Show dialog.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.
		"""
		self.__player.response()
		self.__dialog.show()
		self.__player.response()
		return

	def __hide(self):
		"""
		Hide dialog.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.
		"""
		self.__player.response()
		self.__dialog.hide()
		self.__player.response()
		return

	def __set_properties(self):
		"""
		Set default properties.

		@param self: Reference to the AddDialog instance.
		@type self: An AddDialog object.
		"""
		from gtk import ListStore
		self.__genre_combobox.set_property("model", ListStore(str))
		self.__genre_combobox.set_property("text-column", 0)
		self.__dialog.set_transient_for(self.__manager.glade.get_widget("StationWindow"))
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.
		"""
		from Striim.utils import disconnect_signal, delete_attributes
		disconnect_signal(self.__signal_id_1, self.__manager)
		disconnect_signal(self.__signal_id_2, self.__dialog)
		disconnect_signal(self.__signal_id_3, self.__dialog)
		disconnect_signal(self.__signal_id_4, self.__cancel_button)
		disconnect_signal(self.__signal_id_5, self.__manager)
		disconnect_signal(self.__signal_id_6, self.__save_button)
		from gnomevfs import monitor_cancel
		if self.__monitor_id: monitor_cancel(self.__monitor_id)
		delete_attributes(self)
		del self
		self = None
		return

	def __delete_event_cb(self, *args):
		"""
		Handles callback when the "delete-event" signal is emitted.

		@param self: Reference to the Window instance.
		@type self: A Window object.
		"""
		self.__hide()
		return True

	def __key_press_event_cb(self, window, event):
		"""
		Handles callback when the "key-press-event" signal is emitted.

		@param self: Reference to the Window instance.
		@type self: A Window object.

		@param window: Reference to the Window instance.
		@type window: A Window object.

		@param event: An event.
		@type event: A gtk.Event object.
		"""
		from gtk import keysyms
		from operator import eq
		# Disable tabbing.
		if eq(event.keyval, keysyms.Escape): self.__hide()
		return False

	def __cancel_clicked_cb(self, *args):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.
		"""
		self.__hide()
		return True

	def __save_button_cb(self, *args):
		"""
		Handles callback when the "clicked" signal is emitted.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.
		"""
		self.__update_database()
		self.__hide()
		return

	def __treeview_changed_cb(self, manager, info):
		"""
		Handles callback when the "treeview-changed" signal is emitted.

		@param self: Reference to the EditDialog instance.
		@type self: An EditDialog object.

		@param manager: Reference to the Manager instance.
		@type manager: A Manager object.

		@param info: Radio station information.
		@type info: A Tuple object.
		"""
		self.__info = info
		return

	def __database_changed_cb(self, monitor_uri, info_uri, event_type):
		"""
		Handles callback when the abbreviation database is modified.

		@param self: Reference to the TemplateLoader instance.
		@type self: An TemplateLoader object.

		@param monitor_uri: The uri that is monitored.
		@type monitor_uri: A String object.

		@param info_uri: The uri that is monitored.
		@type info_uri: A String object.

		@param event_type: The type of modification that occured.
		@type event_type: A gnomevfs.MONITOR_EVENT* object.
		"""
		from gnomevfs import MONITOR_EVENT_DELETED
		from gnomevfs import MONITOR_EVENT_CREATED
		from gnomevfs import MONITOR_EVENT_CHANGED
		from operator import contains
		events = [MONITOR_EVENT_CHANGED, MONITOR_EVENT_DELETED, MONITOR_EVENT_CREATED]
		if contains(events, event_type):
			self.__update_genre_combobox()
		return
