# -*- coding: utf-8 -*-
# Copyright © 2007 Lateef Alabi-Oki
#
# This file is part of striim.
#
# striim is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# striim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with striim; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
# USA

"""
This module documents the class that implements the behavior of the
player's notification icon.

@author: Lateef Alabi-Oki
@organization: The Scribes Project
@copyright: Copyright © 2007 Lateef Alabi-Oki
@license: GNU GPLv2 or Later
@contact: mystilleef@gmail.com
"""

from gtk import StatusIcon

class StriimStatusIcon(StatusIcon):
	"""
	This class defines the behavior of the icon in the notification
	area.
	"""

	def __init__(self, manager, player):
		"""
		Initialize object.

		@param self: Reference to the StatusIcon instance.
		@type self: A StatusIcon object.

		@param player: Reference to the Player
		@type player: A Player object.
		"""
		StatusIcon.__init__(self)
		self.__init_attributes(manager, player)
		self.__set_properties()
		self.__signal_id_1 = manager.connect("destroy", self.__destroy_cb)
		self.__signal_id_2 = manager.connect("show-status-window", self.__show_status_window_cb)
		self.__signal_id_3 = self.connect("activate", self.__activate_cb)
		self.__signal_id_4 = self.connect("popup-menu", self.__popup_menu_cb)
		self.__signal_id_5 = self.connect("size-changed", self.__size_changed_cb)
		self.__signal_id_6 = player.connect("stream-update", self.__update_cb)
		from gobject import timeout_add
		timeout_add(1000, self.__broadcast_position)
		from gobject import MainLoop
		MainLoop().run()

	def __init_attributes(self, manager, player):
		"""
		Initialize data attributes.

		@param self: Reference to the StatusIcon instance.
		@type self: A StatusIcon object.

		@param player: Reference to the Player
		@type player: A Player object.
		"""
		from Striim.Global import striim_data_folder
		from os.path import join
		self.__manager = manager
		self.__player = player
		image_file = join(striim_data_folder, "striim.png")
		from Striim.utils import get_pixbuf_from_file
		self.__image_pixbuf = get_pixbuf_from_file(image_file, (22, 22))
		self.__signal_id_1 = self.__signal_id_2 = self.__signal_id_3 = None
		self.__signal_id_4 = self.__signal_id_5 = None
		return

	def __set_properties(self):
		"""
		Set default properties.

		@param self: Reference to the StatusIcon instance.
		@type self: A StatusIcon object.
		"""
		self.set_property("pixbuf", self.__image_pixbuf)
		self.set_property("visible", False)
		if self.is_embedded: self.set_property("visible", True)
		return

	def __activate_cb(self, *args):
		"""
		Handles callback when the "activate" signal is emitted.

		@param self: Reference to the StatusIcon instance.
		@type self: A StatusIcon object.
		"""
		self.__player.trigger("toggle_player_visibility")
		from gobject import idle_add
		idle_add(self.__broadcast_position)
		return True

	def __popup_menu_cb(self, icon, button, time):
		"""
		Handles callback when the "popup-menu" signal is emitted.

		@param self: Reference to the StatusIcon instance.
		@type self: A StatusIcon object.

		@param icon: Reference to the StatusIcon.
		@type icon: A StatusIcon object.

		@param button: Mouse button clicked.
		@type button: A gtk.gdk.Event object.

		@param time: The time the mouse button was clicked.
		@type time: An Integer object.
		"""
		self.__player.response()
		from gobject import idle_add
		idle_add(self.__broadcast_position)		
		self.__player.emit("show-popup", (button, time, self))
		self.__player.response()
		return True

	def __show_status_window_cb(self, *args):
		"""
		Handles callback when the "show-status-window" signal is emitted.

		@param self: Reference to the StatusIcon instance.
		@type self: A StatusIcon object.
		"""
		from gobject import idle_add
		idle_add(self.__broadcast_position)
		return

	def __destroy_cb(self, *args):
		"""
		Handles callback when the "destroy" signal is emitted.

		@param self: Reference to the StatusIcon instance.
		@type self: A StatusIcon object.
		"""
		from Striim.utils import delete_attributes, disconnect_signal
		disconnect_signal(self.__signal_id_1, self.__manager)
		disconnect_signal(self.__signal_id_2, self.__manager)
		disconnect_signal(self.__signal_id_3, self)
		disconnect_signal(self.__signal_id_4, self)
		disconnect_signal(self.__signal_id_5, self)
		disconnect_signal(self.__signal_id_6, self.__player)
		delete_attributes(self)
		del self
		self = None
		return

	def __size_changed_cb(self, *args):
		"""
		Handles callback when the "size-changed" signal is emitted.

		@param self: Reference to the StatusIcon instance.
		@type self: A StatusIcon object.
		"""
		from gobject import idle_add
		idle_add(self.__broadcast_position)
		return

	def __update_cb(self, *args):
		"""
		Handles callback when the "stream-update" signal is emitted.

		@param self: Reference to the StatusIcon instance.
		@type self: A StatusIcon object.
		"""
		from gobject import idle_add
		idle_add(self.__broadcast_position)
		return

	def __broadcast_position(self):
		"""
		Broadcast the position of the status icon.

		@param self: Reference to the StatusIcon instance.
		@type self: A StatusIcon object.
		"""
		try:
			self.__manager.emit("icon-position", self.get_geometry())
		except:
			self.__manager.emit("icon-position", None)
		return False
